import unittest
import StringIO

import MultipartParser

TEST1MESSAGE="""preamble\r
--thisisboundary\r
first part\r
--thisisboundary\r
second part\r
\r
--thisisboundary--\r
postamble"""

TEST2MESSAGE="""preamble\r
--b\rfirstpart
--b"""

TEST3MESSAGE="""\r
--rotfl\r
\r
--rotfl\r
foo\r
bar\r"""

TEST4MESSAGE="""preamble\r"""

TEST5MESSAGE="""\r
--rotfl\r
msg\r
--rotflgarbage\r
--rotfl--\r"""

TEST6MESSAGE="""\r
--rotfl--\r
"""

TEST7MESSAGE="""preamble
--thisisboundary
first part
--thisisboundary
second part

--thisisboundary--
postamble"""

class MultipartParserTestCase(unittest.TestCase):

	def __getParser(self, boundary, message):
		fh = StringIO.StringIO(message)
		return MultipartParser.MultipartParser(fh, boundary)

	def test1(self):
		mp = self.__getParser("thisisboundary", TEST1MESSAGE)

		self.assertEquals(mp.read(1024), "first part")
		self.assertEquals(mp.read(1024), "")

		self.assertEquals(mp.next(), True)

		self.assertEquals(mp.read(1024), "second part\r\n")
		self.assertEquals(mp.read(1024), "")
		self.assertEquals(mp.read(1024), "")

		self.assertEquals(mp.next(), False)

	def test1_1(self):
		mp = self.__getParser("thisisboundary", TEST1MESSAGE)

		self.assertEquals(mp.read(9), "first par")
		self.assertEquals(mp.read(2), "t")
		self.assertEquals(mp.read(2), "")
		self.assertEquals(mp.next(), True)

		self.assertEquals(mp.read(1), "s")
		self.assertEquals(mp.read(1), "e")
		self.assertEquals(mp.read(1), "c")
		self.assertEquals(mp.read(1), "o")
		self.assertEquals(mp.read(1), "n")
		self.assertEquals(mp.read(1), "d")
		self.assertEquals(mp.read(1), " ")
		self.assertEquals(mp.read(1), "p")
		self.assertEquals(mp.read(1), "a")
		self.assertEquals(mp.read(1), "r")
		self.assertEquals(mp.read(1), "t")
		self.assertEquals(mp.read(1), "\r")
		self.assertEquals(mp.read(1), "\n")
		self.assertEquals(mp.read(1), "")

		self.assertEquals(mp.next(), False)

	def test2(self):
		self.assertRaises(MultipartParser.MalformedBoundaryError, self.__getParser, "b", TEST2MESSAGE)

	def test3(self):
		mp = self.__getParser("rotfl", TEST3MESSAGE)

		self.assertEquals(mp.read(1024), "")
		self.assertEquals(mp.next(), True)
		self.assertRaises(MultipartParser.NoBoundaryFoundError, mp.read, 1024)

	def test4(self):
		print "[[test4]]"
		self.assertRaises(MultipartParser.NoBoundaryFoundError, self.__getParser, "b", TEST4MESSAGE)

	def test5(self):
		print "[[test5]]"
		mp = self.__getParser("rotfl", TEST5MESSAGE)
		mp.read(1024)
		self.assertRaises(MultipartParser.MalformedBoundaryError, mp.next)

	def test6(self):
		self.assertRaises(MultipartParser.MalformedBoundaryError, self.__getParser,
						  "rotfl", TEST6MESSAGE)

	def test7(self):
		mp = self.__getParser("thisisboundary", TEST7MESSAGE)
		self.assertEquals(mp.read(1000), "first part")
		self.assertEquals(mp.read(1000), "")
		self.assertEquals(mp.next(), True)
		self.assertEquals(mp.read(1000), "second part\n")
		self.assertEquals(mp.read(1000), "")
		self.assertEquals(mp.next(), False)

	#TODO: more tests for the read(small-value)
	#TODO: tests for data-on-the-edge-of-buffer (should also be tested in test_Buffer)

if __name__ == '__main__':
	unittest.main()
